package com.prosperolambda.android.app.hecate;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Typeface;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.PowerManager;
import android.os.Vibrator;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

public class AlarmaActivity extends Activity implements View.OnClickListener {

	private PowerManager mPowerManager;	
	private PowerManager.WakeLock mWakelock;
	private Button mBoton;
	private TextView mTextoSegundos;
	private ProgressBar mBarra;
	private Vibrator mVibrador;
	private Handler mHandler;
	private Timer mTimer;
	private MediaPlayer mPlayer;
	private boolean mbEstaActivo;
	private boolean mbDebug;
	private long mlTimeout;
	private int miContador;
	private int miVolumen;
	
	private static final String WAKELOCK_TAG="AlarmaActivity_Wakelock";
	private static final String TAG="AlarmaActivity";
	private static final long PATRON_VIBRACION[]= { 500, 2000 };
	private static final long TIMEOUT=90000; //90 segundos de alarma antes de realizar la llamada de emergencia.
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_alarma);
		Log.d(TAG, "Arrancando Alarma");
		
		//Establece la tipografa Roboto en los cuadros de texto
		Typeface tf= Typeface.createFromAsset( getAssets(), "fonts/Roboto-Light.ttf");
		TextView advertencia= (TextView) findViewById(R.id.advertencia);
		mTextoSegundos= (TextView) findViewById(R.id.segundos);
		mTextoSegundos.setTypeface(tf);
		advertencia.setTypeface(tf);
		
		//Asocia el botn con el listener
		mBoton= (Button) findViewById(R.id.boton);
		mBoton.setOnClickListener(this);
		
		//Si est en Modo Debug, el tiemout ser de 5 segundos para comodidad del usuario.
		SharedPreferences preferencias= PreferenceManager.getDefaultSharedPreferences(this);
		mbDebug=preferencias.getBoolean("opciones_debug", false);
		mlTimeout= mbDebug ? 8000 : TIMEOUT;
		
		//Establece los parmetros de la barra de progreso
		mBarra= (ProgressBar) findViewById(R.id.barra_progreso);
		mBarra.setMax( (int)mlTimeout/1000);
		
		//Crea un Wakelock Parcial para que el telfono no entre en suspensin.
		mPowerManager= (PowerManager) getSystemService(Context.POWER_SERVICE);
		mWakelock=mPowerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, WAKELOCK_TAG);
		mWakelock.acquire();
		Log.d(TAG,"Full Wakelock ADQUIRIDO");
		
		/*
		 * FLAG_TURN_SCREEN_ON: Enciende la pantalla nada ms crearse la ventana.
		 * FLAG_KEEP_SCREEN_ON: Impide que la pantalla se apague.
		 */
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
					|WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		
		/*
		 * FLAG_DISMISS_KEYGUARD y
		 * FLAG_SHOW_WHEN_LOCKED Intenta en la medida de lo posible
		 * que la ventana se muestre ignorando el bloqueo de pantalla.
		 */
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
					|WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);
		Log.d(TAG,"Pantalla ENCENDIDA");
		
		/*
		 * Vibra usando el patrn definidio como constante (vibra dos segundos, se apaga medio segundo) 
		 * repitindose hasta que se desactiva o salta el timeout.
		 */
		mVibrador= (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
		if(mVibrador != null)
			mVibrador.vibrate(PATRON_VIBRACION, 0);
		
		/*
		 * Empieza a reproducir la sirena de forma asncrona, en modo repeticin ininterrumpidamente
		 * hasta que salte el timeout o el usuario la apague. Ajusta el volmen de reproduccin al mximo.
		 * Al finalizar restablece el volmen a sy valor original
		 */
		AudioManager audio = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
		miVolumen= audio.getStreamVolume(AudioManager.STREAM_MUSIC);
		Log.d(TAG, "Volumen inicial "+miVolumen);
		
		audio.setStreamVolume(AudioManager.STREAM_MUSIC, 
				audio.getStreamMaxVolume(AudioManager.STREAM_MUSIC), 0);
		Log.d(TAG, "Volumen al mximo");
		
		mPlayer= MediaPlayer.create(this, R.raw.sirena);
		mPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC);
		mPlayer.setLooping(true);
		mPlayer.start();
		Log.d(TAG, "Reproduciendo msica");
		
		
		mbEstaActivo=true;
		miContador=0; //Segundos para timeout
		
		/*
		 * Los mensajes enviados por Timer son recogidos por el mtodo handleMessage de handler.
		 * Cada vez que se reciba un mensaje se proceder a actualizar la interfaz grfica.
		 */
		mHandler=new Handler(){
			@Override
			public void handleMessage(Message msg){
				Resources res= getResources();
				String texto= new String(miContador+"\n"+res.getString(R.string.alarma_segundos));
				
				mTextoSegundos.setText( texto );
				mBarra.setProgress(miContador);
				
				if(miContador*1000 >= mlTimeout){
					mTimer.cancel();
					pararAlarma();
					UrgenciaDispatcher.getInstance().activarEmergencia();
				}
			}
		};
		
		/*
		 * Ejecutndose cada segundo en un hilo aparte. TimerTask aumenta en uno el contador y acto seguido enva
		 * un mensaje al Handler.
		 */
		mTimer= new Timer();
		mTimer.scheduleAtFixedRate(new TimerTask(){
			@Override
			public void run(){

				miContador++;
				mHandler.sendEmptyMessage(0);
				Log.d(TAG, "Actualizando contador: "+miContador+ " segundos");
			}
		}, 1000, 1000);
	}


	@Override
	protected void onDestroy() {
		super.onDestroy();
		Log.d(TAG, "Destruyendo AlarmaActivity");
		//pararAlarma();
	}
	
	/*
	 * Sobreescribiendo este mtodo se impide que el usuario salga accidentalmente de la
	 * actividad sin haber desactivado explcitamente la Alarma.
	 */
	@Override
	public void onBackPressed(){
		if(mbEstaActivo){
			Toast toast=Toast.makeText(this, R.string.alarma_toast, Toast.LENGTH_SHORT);
			toast.show();
		}
		else{
			super.onBackPressed();
			return;
		}
	}
	@Override
	public void onClick(View v) {
		pararAlarma();	
	}
	
	private void pararAlarma(){
		mbEstaActivo=false;
		
		if(mWakelock != null){
			mWakelock.release();
			mWakelock=null;
			Log.d(TAG, "Partial Wakelock LIBERADO");
		}
		
		if(mPlayer != null){
			mPlayer.release();
			Log.d(TAG, "Apagando la Sirena");
			mPlayer=null;
			
			AudioManager audio= (AudioManager) getSystemService(Context.AUDIO_SERVICE);
			audio.setStreamVolume(AudioManager.STREAM_MUSIC, miVolumen, 0);
			Log.d(TAG, "Volumen original restablecido");
		}
		
		mVibrador.cancel();
		Log.d(TAG, "Vibracin detenida");
		
		mTimer.cancel();
		Log.d(TAG, "Timer Detenido");
	}

}
