package com.prosperolambda.android.app.hecate;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.media.AudioManager;
import android.net.Uri;
import android.os.IBinder;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.telephony.PhoneStateListener;
import android.telephony.TelephonyManager;
import android.util.Log;

public class LlamadaService extends Service implements TextToSpeech.OnUtteranceCompletedListener{
	
	private Timer mTimer;
	private AudioManager mAudio;
	private TelephonyManager mPhone;
	private PhoneStateListener mPhoneListener;
	private String msNumero;
	private String msMensaje;
	private float mPrecision;
	private double[] mPosicion;
	private int miVolumen;
	private boolean mbLlamadaIniciada;
	private TextToSpeech mTTS;
	private HashMap<String,String> mParametros;

	//private static final int ID=1235; //Identificador nico de Servicio	
	private static final String TAG="LlamadaUrgencia";
	private static final String TEXTO_FINALIZADO="Reproduccin TTS finalizada";
			
	@Override
	public void onCreate() {
		super.onCreate();
		Log.d(TAG, "Creando LLAMADASERVICE");
		
		msNumero=UrgenciaDispatcher.getInstance().getNumero();
		mPrecision=UrgenciaDispatcher.getInstance().getPrecision();
		mPosicion=UrgenciaDispatcher.getInstance().getPosicion();
		
		//Establece el volumen de reproduccin del sintetizador al mximo.
		mAudio= (AudioManager) getSystemService(Context.AUDIO_SERVICE);			
		miVolumen=mAudio.getStreamVolume(AudioManager.STREAM_MUSIC);
		Log.d(TAG, "Audio Stream Music= "+miVolumen);
		mAudio.setStreamVolume(AudioManager.STREAM_MUSIC, 
				mAudio.getStreamMaxVolume(AudioManager.STREAM_MUSIC), 0);
		Log.d(TAG, "Audio Stream Music MAXIMO= "+mAudio.getStreamVolume(AudioManager.STREAM_MUSIC));
		
		//Establece los parmetros que sern usados por el motor de sintetizador de texto
		mParametros= new HashMap<String, String>();
		//La salida del sintetizador sea el altavoz externo.
		mParametros.put(TextToSpeech.Engine.KEY_PARAM_STREAM, 
				String.valueOf(AudioManager.STREAM_MUSIC) );
		//Establece el listener que ser invocado cada vez que termine la reproduccin actual
		mParametros.put(TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID, TEXTO_FINALIZADO);
		
		mPhone= (TelephonyManager) getSystemService(Context.TELEPHONY_SERVICE);
		mTimer=new Timer();
		
		//Se conforma el mensaje que ser reproducido por voz
		String latitud= Location.convert(mPosicion[0], Location.FORMAT_DEGREES);
		String longitud= Location.convert(mPosicion[1], Location.FORMAT_DEGREES);
		msMensaje= getString(R.string.urgencia_llamada_mensaje, (int)mPrecision, latitud, longitud);
		Log.d(TAG, "TTS reproducira: "+msMensaje);
		
		mPhoneListener=new PhoneStateListener(){
			@Override
			public void onCallStateChanged(int state, String incomingNumer){
				
				switch(state){
				case TelephonyManager.CALL_STATE_IDLE:
					Log.d(TAG, "Telfono en estado IDLE");
					if(mbLlamadaIniciada){
						mPhone.listen(mPhoneListener, PhoneStateListener.LISTEN_NONE);
						
						mAudio.setStreamVolume(AudioManager.STREAM_MUSIC, miVolumen, 0);
						Log.d(TAG, "Audio al salir del servicio= "+mAudio.getStreamVolume(AudioManager.STREAM_MUSIC));
						stopSelf();
					}	
					break;
				
				case TelephonyManager.CALL_STATE_OFFHOOK:
					mbLlamadaIniciada=true;
					Log.d(TAG, "Telfono en estado LLAMANDO");
					//TODO JUSTIFICAR MEMORIA IMPORTANTE
					//DEBUG
					try{
						Class audioSystemClass = Class.forName("android.media.AudioSystem");
						Method setForceUse = audioSystemClass.getMethod("setForceUse",
						    int.class,
						    int.class);
						// First 0 == FOR_COMMUNICATION, second 0 == FORCE_NONE.
						Log.d(TAG, "Modo ANTES de forzar "+mAudio.getMode() );
						setForceUse.invoke(null, 0, 1);
					}catch(Exception e){ e.printStackTrace(); }
					
					//Se espera 5 segundos y se empieza la primera reproduccin.
					mTimer.schedule(new TimerTask(){
						@Override
						public void run(){
							mTTS.setSpeechRate(0.75f);
							mTTS.speak(msMensaje, TextToSpeech.QUEUE_ADD, mParametros);
						}
					}, 7000);
					break;
				
				case TelephonyManager.CALL_STATE_RINGING:
					Log.d(TAG, "Telfono en estado RINGING");
					break;
				
				default:
					Log.d(TAG, "WTF, no debera entrar aqu!");
				}
			};
		};
			
	}
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		mTTS= new TextToSpeech(this, new OnInitListener() {			
			@SuppressWarnings("deprecation")
			@Override
			public void onInit(int status) {
				mTTS.setOnUtteranceCompletedListener(LlamadaService.this);
								
				//Una vez iniciado el motor TTS, se procede a llamar por tlf
				String llamada="tel:"+msNumero.trim();
				Intent intent= new Intent(Intent.ACTION_CALL,Uri.parse(llamada));
				intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				startActivity(intent);					
			}
		});	

		mPhone.listen(mPhoneListener , PhoneStateListener.LISTEN_CALL_STATE);							
		return Service.START_STICKY;
	}
	
	@Override
	public void onUtteranceCompleted(String utteranceId) {
		mTTS.speak(msMensaje, TextToSpeech.QUEUE_ADD, mParametros);			
	}
	
	@Override
	public void onDestroy() {
		Log.d(TAG, "Destruyendo LlamadaService");
		super.onDestroy();
		mTTS.stop();
		mTTS.shutdown();
		//TODO JUSTIFICAR MEMORIA IMPORTANTE2
		//DEBUG
		try{
			Class audioSystemClass = Class.forName("android.media.AudioSystem");
			Method setForceUse = audioSystemClass.getMethod("setForceUse",
			    int.class,
			    int.class);
			// First 0 == FOR_COMMUNICATION, second 0 == FORCE_NONE.
			setForceUse.invoke(null, 0, 0);
		}catch(Exception e){ e.printStackTrace(); }
		Log.d(TAG, "SetForce reseteado a valores originales!");
		Log.d(TAG, "Servicio Destrudo");
	}

	//No se usa en esta aplicacin
	@Override
	public IBinder onBind(Intent arg0) { return null; }
	
}