package com.prosperolambda.android.app.hermes;

import java.io.File;
import java.util.Calendar;
import java.util.Locale;

import android.app.Service;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;

public class ServicioRegistroSensores extends Service implements SensorEventListener, Callback{

	private Sensor mSensor;
	private SensorManager mSensorManager;	
	private Messenger mActivity;
	private Messenger mServicio;
	
	private FicheroLogger mLogger;
	private String[] msBuffer;
	
	private int miPosicion;
	
	private float[] mfMaximas;
	
	public static final int TAM_BUFFER=2000;
	public static final int MENSAJE_CONECTAR=1;
	public static final int MENSAJE_DESCONECTAR=2;
	public static final int MENSAJE_VALORES=3;
	
	public final static String TAG="ServicioRegistroSensores";
	
	@Override
	public void onCreate() {
		super.onCreate();
		
		mServicio=new Messenger(new Handler(this) );
		Log.d(TAG, "Servicio creado");
		//Se inicializa el buffer
		msBuffer= new String[TAM_BUFFER];
		msBuffer[0]= String.format("Registro Acelerometro: \n" +
				" %1$tA %1$tm de %1$tB, %1$tY:\n\n", Calendar.getInstance() );
		
		miPosicion=1;
		mfMaximas= new float[3];		
				
		mSensorManager= (SensorManager) this.getSystemService(SENSOR_SERVICE);
		mSensor= mSensorManager.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION);
		
		mLogger= new FicheroLogger( new File(getExternalFilesDir(null), 
				getString(R.string.ficheroLog)) );
		
		Log.d(TAG, "Servicio Arrancado");
		mSensorManager.registerListener(this, mSensor, SensorManager.SENSOR_DELAY_UI);

	}
	
	@Override
	public void onDestroy() {

		// Primero de todo, "desregistra" el listener.
		mSensorManager.unregisterListener(this);
		// Vuelca cualquier contenido pendiente de loggear
		// y mata el hilo.
		mLogger.agregarTraza(msBuffer);
		mLogger.desactivar();
		/*
		 * Antes de ser destruido, el servicio enva un mensaje de vuelta con
		 * los valores mximos de las aceleraciones registradas.
		 */
		Message mensaje = new Message();
		mensaje.what = MENSAJE_VALORES;
		mensaje.obj = mfMaximas;
		try {
			mActivity.send(mensaje);
		} catch (RemoteException e) {
			Log.e(TAG, "Error enviando el mensaje VALORES");
		}
		super.onDestroy();
		Log.d(TAG, "Destruyendo el servicio");
	}
	
	@Override
	public IBinder onBind(Intent arg0) {
		return mServicio.getBinder();
	}

	@Override
	public boolean onUnbind(Intent intent) {
		return true;
	}
	
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		
	}

	public void onSensorChanged(SensorEvent event) {		
		
		if(event.values[0] > mfMaximas[0]) mfMaximas[0]=event.values[0];
		if(event.values[1] > mfMaximas[1]) mfMaximas[1]=event.values[1];
		if(event.values[2] > mfMaximas[2]) mfMaximas[2]=event.values[2];
		
		/*
		 * Si se ha llenado el buffer, se agrega al logger y 
		 * se procede a llenar uno nuevo
		 */
		if(miPosicion == TAM_BUFFER-1){
			mLogger.agregarTraza(msBuffer);
			msBuffer=new String[TAM_BUFFER];
			miPosicion=0;
		}
		/*
		 * Agrega los valores al buffer con el formato:
		 * 
		 * valor_ejeX  valor_ejeY  valor_ejeZ   [HH:mm:ss.mmm]
		 * 
		 * Valores expresados en m/s^2 con 2 decimales (usando el caracter punto para los decimales)
		 * Aceleraciones Expresadas en valores absolutos.
		 */
		msBuffer[miPosicion]= String.format(Locale.US, "%1$06.2f\t%2$06.2f\t%3$06.2f\t[%4$tT.%4$tL]\n", 
				Math.abs(event.values[0]), Math.abs(event.values[1]), Math.abs(event.values[2]),
				Calendar.getInstance() );
		miPosicion++;
	}

	public boolean handleMessage(Message mensaje) {
		Log.d(TAG, "RECIBE UN MENSAJITO!");
		/*
		 * Recibe mensaje por parte de la actividad, as ya existe un destino
		 * al que enviar el resultado una vez que el destino se pare.
		 */
		switch(mensaje.what){
			case MENSAJE_CONECTAR:
				mActivity=mensaje.replyTo;
				return true;
			
			default:
				Log.d(TAG, "ENTRA EN EL DEFAULT??");
				break;
		}
		
		return false;
	}

}
